@@ -325,7 +325,7 @@ _add_fuse_mount (xlator_t *graph)
         if (cmd_args->fuse_attribute_timeout >= 0)
                 ret = dict_set_double (top->options, ZR_ATTR_TIMEOUT_OPT,
                                        cmd_args->fuse_attribute_timeout);
-        if (cmd_args->fuse_entry_timeout)
+        if (cmd_args->fuse_entry_timeout >= 0)
                 ret = dict_set_double (top->options, ZR_ENTRY_TIMEOUT_OPT,
                                        cmd_args->fuse_entry_timeout);
 
@@ -1157,6 +1157,7 @@ main (int argc, char *argv[])
         cmd_args->log_level = DEFAULT_LOG_LEVEL;
         cmd_args->fuse_direct_io_mode_flag = 2;
         cmd_args->fuse_attribute_timeout = -1;
+        cmd_args->fuse_entry_timeout = -1;
 
         INIT_LIST_HEAD (&cmd_args->xlator_options);
 
