@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2006-2009 Z RESEARCH, Inc. <http://www.zresearch.com>
+   Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
@@ -32,4 +32,6 @@ uint32_t SuperFastHash (const char * data, int32_t len);
 
 uint32_t gf_dm_hashfn (const char *msg, int len);
 
+uint32_t
+ReallySimpleHash (char *path, int len);
 #endif /* __HASHFN_H__ */
