@@ -1,4 +1,4 @@
-/*
+/* 
   Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
@@ -41,6 +41,8 @@ typedef struct _call_pool_t call_pool_t;
 #include "dict.h"
 #include "list.h"
 #include "common-utils.h"
+#include "globals.h"
+#include "protocol.h"
 
 
 typedef int32_t (*ret_fn_t) (call_frame_t *frame,
@@ -73,6 +75,7 @@ struct _call_frame_t {
 	int32_t       ref_count;
 	gf_lock_t     lock;
 	void         *cookie;      /* unique cookie */
+	gf_boolean_t  complete;
 };
 
 struct _call_stack_t {
@@ -90,6 +93,10 @@ struct _call_stack_t {
 	uid_t                         uid;
 	gid_t                         gid;
 	pid_t                         pid;
+        uint32_t                      ngrps;
+        uint32_t                      groups[GF_REQUEST_MAXGROUPS];
+        uint64_t                      lk_owner;
+
 	call_frame_t                  frames;
 
 	int32_t                       op;
@@ -140,6 +147,7 @@ STACK_DESTROY (call_stack_t *stack)
 #define STACK_WIND(frame, rfn, obj, fn, params ...)			\
 	do {								\
 		call_frame_t *_new = NULL;				\
+                xlator_t     *old_THIS = NULL;                          \
 		                                                        \
                 _new = CALLOC (1, sizeof (call_frame_t));	        \
 		ERR_ABORT (_new);					\
@@ -157,15 +165,20 @@ STACK_DESTROY (call_stack_t *stack)
 		LOCK_INIT (&_new->lock);				\
 		frame->ref_count++;					\
 									\
+                old_THIS = THIS;                                        \
+                THIS = obj;                                             \
 		fn (_new, obj, params);					\
+                THIS = old_THIS;                                        \
 	} while (0)
 
 
 /* make a call with a cookie */
 #define STACK_WIND_COOKIE(frame, rfn, cky, obj, fn, params ...)		\
 	do {								\
-		call_frame_t *_new = CALLOC (1,				\
-					     sizeof (call_frame_t));	\
+                call_frame_t *_new = NULL;                              \
+                xlator_t     *old_THIS = NULL;                          \
+                                                                        \
+                _new = CALLOC (1, sizeof (call_frame_t));               \
 		ERR_ABORT (_new);					\
 		typeof(fn##_cbk) tmp_cbk = rfn;				\
 		_new->root = frame->root;				\
@@ -182,17 +195,46 @@ STACK_DESTROY (call_stack_t *stack)
 		frame->ref_count++;					\
 		fn##_cbk = rfn;						\
 									\
+                old_THIS = THIS;                                        \
+                THIS = obj;                                             \
 		fn (_new, obj, params);					\
+                THIS = old_THIS;                                        \
 	} while (0)
 
 
 /* return from function */
 #define STACK_UNWIND(frame, params ...)					\
 	do {								\
-		ret_fn_t fn = frame->ret;				\
-		call_frame_t *_parent = frame->parent;			\
+		ret_fn_t      fn = NULL;                                \
+		call_frame_t *_parent = NULL;                           \
+                xlator_t     *old_THIS = NULL;                          \
+                                                                        \
+                fn = frame->ret;                                        \
+                _parent = frame->parent;                                \
 		_parent->ref_count--;					\
+                old_THIS = THIS;                                        \
+                THIS = _parent->this;                                   \
+                frame->complete = _gf_true;                             \
 		fn (_parent, frame->cookie, _parent->this, params);	\
+                THIS = old_THIS;                                        \
+	} while (0)
+
+
+/* return from function in type-safe way */
+#define STACK_UNWIND_STRICT(op, frame, params ...)                      \
+	do {								\
+		fop_##op##_cbk_t      fn = NULL;                        \
+		call_frame_t *_parent = NULL;                           \
+                xlator_t     *old_THIS = NULL;                          \
+                                                                        \
+                fn = (fop_##op##_cbk_t )frame->ret;                     \
+                _parent = frame->parent;                                \
+		_parent->ref_count--;					\
+                old_THIS = THIS;                                        \
+                THIS = _parent->this;                                   \
+                frame->complete = _gf_true;                             \
+		fn (_parent, frame->cookie, _parent->this, params);	\
+                THIS = old_THIS;                                        \
 	} while (0)
 
 
@@ -207,16 +249,24 @@ copy_frame (call_frame_t *frame)
 	}
 
 	newstack = (void *) CALLOC (1, sizeof (*newstack));
+        if (newstack == NULL) {
+                return NULL;
+        }
+
 	oldstack = frame->root;
 
 	newstack->uid = oldstack->uid;
 	newstack->gid = oldstack->gid;
 	newstack->pid = oldstack->pid;
+        newstack->ngrps = oldstack->ngrps;
+        memcpy (newstack->groups, oldstack->groups,
+                sizeof (uint32_t) * GF_REQUEST_MAXGROUPS);
 	newstack->unique = oldstack->unique;
 
 	newstack->frames.this = frame->this;
 	newstack->frames.root = newstack;
 	newstack->pool = oldstack->pool;
+        newstack->lk_owner = oldstack->lk_owner;
 
 	LOCK_INIT (&newstack->frames.lock);
 
@@ -260,5 +310,7 @@ create_frame (xlator_t *xl, call_pool_t *pool)
 	return &stack->frames;
 }
 
+void
+gf_proc_dump_pending_frames(call_pool_t *call_pool);
 
 #endif /* _STACK_H */
