@@ -111,8 +111,6 @@ typedef struct {
                                            for the duration specified in this
                                            member. In Seconds.
                                            */
-
-        int            relativepaths;
 } glusterfs_init_params_t;
 
 
@@ -392,9 +390,8 @@ glusterfs_setxattr (const char *path, const char *name, const void *value,
                         size_t size, int flags);
 
 int
-glusterfs_lsetxattr (glusterfs_handle_t handle, const char *path,
-                     const char *name, const void *value, size_t size,
-                     int flags);
+glusterfs_lsetxattr (const char *path, const char *name, const void *value,
+                     size_t size, int flags);
 
 int
 glusterfs_fsetxattr (glusterfs_file_t fd, const char *name, const void *value,
@@ -1307,6 +1304,52 @@ glusterfs_sendfile (int out_fd, glusterfs_file_t in_fd, off_t *offset,
 int
 glusterfs_fcntl (glusterfs_file_t fd, int cmd, ...);
 
+/*
+ * Change the current working directory to @path
+ * 
+ * @path  : path to change the current working directory to.
+ *
+ * Returns 0 on success and -1 on failure with errno set appropriately.
+ */
+int
+glusterfs_chdir (const char *path);
+
+/*
+ * Change the current working directory to the path @fd is opened on.
+ *
+ * @fd   : current working directory will be changed to path @fd is opened on.
+ *
+ * Returns 0 on success and -1 on  with errno set appropriately.
+ */
+int
+glusterfs_fchdir (glusterfs_file_t fd);
+
+/* copies the current working directory into @buf if it is big enough
+ *
+ * @buf: buffer to copy into it. If @buf is NULL, a buffer will be allocated.
+ *       The size of the buffer will be @size if it is not zero, otherwise the
+ *       size will be big enough to hold the current working directory.
+ * @size: size of the buffer.
+ *
+ * Returns the pointer to buffer holding current working directory on success
+ * and NULL on failure.
+ */
+
+char *
+glusterfs_getcwd (char *buf, size_t size);
+
+/*
+ * Truncate the file to a specified length.
+ *
+ * @path   : path to the file.
+ * @length : length to which the file has to be truncated.
+ *
+ * Returns 0 on success and -1 on failure with errno set appropriately 
+ */
+
+int
+glusterfs_truncate (const char *path, off_t length);
+
 
 /* FIXME: review the need for these apis */
 /* added for log related initialization in booster fork implementation */
