@@ -1496,7 +1496,9 @@ afr_self_heal_completion_cbk (call_frame_t *bgsh_frame, xlator_t *this)
         sh    = &local->self_heal;
 
 	if (local->govinda_gOvinda) {
-                afr_set_split_brain (this, local->cont.lookup.inode);
+                afr_set_split_brain (this, local->cont.lookup.inode, _gf_true);
+	} else {
+                afr_set_split_brain (this, local->cont.lookup.inode, _gf_false);
 	}
 
         gf_log (this->name, GF_LOG_TRACE,
@@ -1534,6 +1536,8 @@ afr_self_heal (call_frame_t *frame, xlator_t *this)
 	local = frame->local;
 	priv  = this->private;
 
+        afr_set_lk_owner (frame, this);
+
         if (local->self_heal.background) {
                 LOCK (&priv->lock);
                 {
