@@ -613,7 +613,8 @@ afr_unlock (call_frame_t *frame, xlator_t *this)
                                 call_count--;
                         }
 
-                        if (local->transaction.locked_nodes[i] & LOCKED_YES) {
+                        if (call_count &&
+                             local->transaction.locked_nodes[i] & LOCKED_YES) {
                                 STACK_WIND (frame, afr_unlock_common_cbk,	
                                             priv->children[i], 
                                             priv->children[i]->fops->entrylk, 
@@ -1372,9 +1373,11 @@ int afr_lock_rec (call_frame_t *frame, xlator_t *this, int child_index)
 
 int32_t afr_lock (call_frame_t *frame, xlator_t *this)
 {
+
         afr_pid_save (frame);
 
         frame->root->pid = (long) frame->root;
+        afr_set_lk_owner (frame, this);
 
 	return afr_lock_rec (frame, this, 0);
 }
