@@ -60,6 +60,17 @@
 #define AFR_ICTX_READ_CHILD_MASK       0x00000000FFFFFFFFULL
 
 
+void
+afr_set_lk_owner (call_frame_t *frame, xlator_t *this)
+{
+        if (!frame->root->lk_owner) {
+                gf_log (this->name, GF_LOG_TRACE,
+                        "Setting lk-owner=%llu",
+                        (unsigned long long) frame->root);
+                frame->root->lk_owner = (uint64_t) frame->root;
+        }
+}
+
 uint64_t
 afr_is_split_brain (xlator_t *this, inode_t *inode)
 {
@@ -88,7 +99,7 @@ out:
 
 
 void
-afr_set_split_brain (xlator_t *this, inode_t *inode)
+afr_set_split_brain (xlator_t *this, inode_t *inode, gf_boolean_t set)
 {
         uint64_t ctx = 0;
         int      ret = 0;
@@ -103,9 +114,12 @@ afr_set_split_brain (xlator_t *this, inode_t *inode)
                         ctx = 0;
                 }
 
-                ctx = (~AFR_ICTX_SPLIT_BRAIN_MASK & ctx)
-                        | (0xFFFFFFFFFFFFFFFFULL & AFR_ICTX_SPLIT_BRAIN_MASK);
-
+                if (set) {
+			ctx = (~AFR_ICTX_SPLIT_BRAIN_MASK & ctx)
+                                | (0xFFFFFFFFFFFFFFFFULL & AFR_ICTX_SPLIT_BRAIN_MASK);
+		} else {
+			ctx = (~AFR_ICTX_SPLIT_BRAIN_MASK & ctx);
+		}
                 __inode_ctx_put (inode, this, ctx);
         }
         UNLOCK (&inode->lock);
@@ -501,7 +515,7 @@ afr_self_heal_lookup_unwind (call_frame_t *frame, xlator_t *this)
 	local = frame->local;
 
 	if (local->govinda_gOvinda) {
-                afr_set_split_brain (this, local->cont.lookup.inode);
+                afr_set_split_brain (this, local->cont.lookup.inode, _gf_true);
 	}
 
 	AFR_STACK_UNWIND (lookup, frame, local->op_ret, local->op_errno,
@@ -2380,13 +2394,13 @@ afr_lk_unlock (call_frame_t *frame, xlator_t *this)
 
 	if (call_count == 0) {
 		AFR_STACK_UNWIND (lk, frame, local->op_ret, local->op_errno,
-				  &local->cont.lk.flock);
+				  &local->cont.lk.ret_flock);
 		return 0;
 	}
 
 	local->call_count = call_count;
 
-	local->cont.lk.flock.l_type = F_UNLCK;
+	local->cont.lk.user_flock.l_type = F_UNLCK;
 
 	for (i = 0; i < priv->child_count; i++) {
 		if (local->cont.lk.locked_nodes[i]) {
@@ -2394,7 +2408,7 @@ afr_lk_unlock (call_frame_t *frame, xlator_t *this)
 				    priv->children[i],
 				    priv->children[i]->fops->lk,
 				    local->fd, F_SETLK, 
-				    &local->cont.lk.flock);
+				    &local->cont.lk.user_flock);
 
 			if (!--call_count)
 				break;
@@ -2434,6 +2448,7 @@ afr_lk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		local->op_ret        = 0;
 		local->op_errno      = 0;
 		local->cont.lk.locked_nodes[child_index] = 1;
+		local->cont.lk.ret_flock = *lock;
 	}
 
 	child_index++;
@@ -2443,16 +2458,16 @@ afr_lk_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 				   priv->children[child_index],
 				   priv->children[child_index]->fops->lk,
 				   local->fd, local->cont.lk.cmd, 
-				   &local->cont.lk.flock);
+				   &local->cont.lk.user_flock);
 	} else if (local->op_ret == -1) {
 		/* all nodes have gone down */
 		
-		AFR_STACK_UNWIND (lk, frame, -1, ENOTCONN, &local->cont.lk.flock);
+		AFR_STACK_UNWIND (lk, frame, -1, ENOTCONN, &local->cont.lk.ret_flock);
 	} else {
 		/* locking has succeeded on all nodes that are up */
 		
 		AFR_STACK_UNWIND (lk, frame, local->op_ret, local->op_errno,
-                                  lock);
+                                  &local->cont.lk.ret_flock);
 	}
 
 	return 0;
@@ -2494,7 +2509,8 @@ afr_lk (call_frame_t *frame, xlator_t *this,
 
 	local->fd            = fd_ref (fd);
 	local->cont.lk.cmd   = cmd;
-	local->cont.lk.flock = *flock;
+	local->cont.lk.user_flock = *flock;
+        local->cont.lk.ret_flock = *flock;
 
 	STACK_WIND_COOKIE (frame, afr_lk_cbk, (void *) (long) 0,
 			   priv->children[i],
