@@ -269,7 +269,8 @@ typedef struct _afr_local {
 
 		struct {
 			int32_t cmd;
-			struct flock flock;
+			struct flock user_flock;
+                        struct flock ret_flock;
 			unsigned char *locked_nodes;
 		} lk;
 
@@ -579,6 +580,9 @@ typedef struct {
 /* have we tried all children? */
 #define all_tried(i, count)  ((i) == (count) - 1)
 
+void
+afr_set_lk_owner (call_frame_t *frame, xlator_t *this);
+
 int
 afr_fd_ctx_set (xlator_t *this, fd_t *fd);
 
@@ -613,7 +617,7 @@ uint64_t
 afr_is_split_brain (xlator_t *this, inode_t *inode);
 
 void
-afr_set_split_brain (xlator_t *this, inode_t *inode);
+afr_set_split_brain (xlator_t *this, inode_t *inode, gf_boolean_t set);
 
 int
 afr_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
