@@ -48,75 +48,41 @@ map_stat_cbk (call_frame_t *frame,
 	return 0;
 }
 
-
 static int32_t
-map_chmod_cbk (call_frame_t *frame,
-		   void *cookie,
-		   xlator_t *this,
-		   int32_t op_ret,
-		   int32_t op_errno,
-		   struct stat *buf)
+map_setattr_cbk (call_frame_t *frame,
+                 void *cookie,
+                 xlator_t *this,
+                 int32_t op_ret,
+                 int32_t op_errno,
+                 struct stat *statpre,
+                 struct stat *statpost)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
-	
-	map_itransform (this, prev->this, buf->st_ino, &buf->st_ino);
 
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
+	map_itransform (this, prev->this, statpre->st_ino, &statpre->st_ino);
+	map_itransform (this, prev->this, statpost->st_ino, &statpost->st_ino);
+
+	STACK_UNWIND (frame, op_ret, op_errno, statpre, statpost);
 	return 0;
 }
 
-
 static int32_t
-map_fchmod_cbk (call_frame_t *frame,
-		    void *cookie,
-		    xlator_t *this,
-		    int32_t op_ret,
-		    int32_t op_errno,
-		    struct stat *buf)
+map_fsetattr_cbk (call_frame_t *frame,
+                  void *cookie,
+                  xlator_t *this,
+                  int32_t op_ret,
+                  int32_t op_errno,
+                  struct stat *statpre,
+                  struct stat *statpost)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
-	
-	map_itransform (this, prev->this, buf->st_ino, &buf->st_ino);
 
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
-	return 0;
-}
+	map_itransform (this, prev->this, statpre->st_ino, &statpre->st_ino);
+	map_itransform (this, prev->this, statpost->st_ino, &statpost->st_ino);
 
-
-static int32_t
-map_chown_cbk (call_frame_t *frame,
-		   void *cookie,
-		   xlator_t *this,
-		   int32_t op_ret,
-		   int32_t op_errno,
-		   struct stat *buf)
-{
-        call_frame_t *prev = NULL;
-        prev  = cookie;
-	
-	map_itransform (this, prev->this, buf->st_ino, &buf->st_ino);
-
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
-	return 0;
-}
-
-
-static int32_t
-map_fchown_cbk (call_frame_t *frame,
-		    void *cookie,
-		    xlator_t *this,
-		    int32_t op_ret,
-		    int32_t op_errno,
-		    struct stat *buf)
-{
-        call_frame_t *prev = NULL;
-        prev  = cookie;
-	
-	map_itransform (this, prev->this, buf->st_ino, &buf->st_ino);
-
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
+	STACK_UNWIND (frame, op_ret, op_errno, statpre, statpost);
 	return 0;
 }
 
@@ -126,14 +92,15 @@ map_truncate_cbk (call_frame_t *frame,
 		      xlator_t *this,
 		      int32_t op_ret,
 		      int32_t op_errno,
-		      struct stat *buf)
+		      struct stat *prebuf,
+                      struct stat *postbuf)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
 	
-	map_itransform (this, prev->this, buf->st_ino, &buf->st_ino);
+	map_itransform (this, prev->this, postbuf->st_ino, &postbuf->st_ino);
 
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
+	STACK_UNWIND (frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -143,31 +110,15 @@ map_ftruncate_cbk (call_frame_t *frame,
 		       xlator_t *this,
 		       int32_t op_ret,
 		       int32_t op_errno,
-		       struct stat *buf)
+		       struct stat *prebuf,
+                       struct stat *postbuf)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
 	
-	map_itransform (this, prev->this, buf->st_ino, &buf->st_ino);
+	map_itransform (this, prev->this, postbuf->st_ino, &postbuf->st_ino);
 
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
-	return 0;
-}
-
-int32_t
-map_utimens_cbk (call_frame_t *frame,
-		     void *cookie,
-		     xlator_t *this,
-		     int32_t op_ret,
-		     int32_t op_errno,
-		     struct stat *buf)
-{
-        call_frame_t *prev = NULL;
-        prev  = cookie;
-	
-	map_itransform (this, prev->this, buf->st_ino, &buf->st_ino);
-
-	STACK_UNWIND (frame, op_ret, op_errno, buf);
+	STACK_UNWIND (frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -189,9 +140,10 @@ map_readlink_cbk (call_frame_t *frame,
 		      xlator_t *this,
 		      int32_t op_ret,
 		      int32_t op_errno,
-		      const char *path)
+		      const char *path,
+                      struct stat *sbuf)
 {
-	STACK_UNWIND (frame, op_ret, op_errno, path);
+	STACK_UNWIND (frame, op_ret, op_errno, path, sbuf);
 	return 0;
 }
 
@@ -200,9 +152,11 @@ map_unlink_cbk (call_frame_t *frame,
 		    void *cookie,
 		    xlator_t *this,
 		    int32_t op_ret,
-		    int32_t op_errno)
+		    int32_t op_errno,
+                    struct stat *preparent,
+                    struct stat *postparent)
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND (frame, op_ret, op_errno, preparent, postparent);
 	return 0;
 }
 
@@ -211,9 +165,11 @@ map_rmdir_cbk (call_frame_t *frame,
 		   void *cookie,
 		   xlator_t *this,
 		   int32_t op_ret,
-		   int32_t op_errno)
+		   int32_t op_errno,
+                   struct stat *preparent,
+                   struct stat *postparent)
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND (frame, op_ret, op_errno, preparent, postparent);
 	return 0;
 }
 
@@ -224,7 +180,11 @@ map_rename_cbk (call_frame_t *frame,
 		    xlator_t *this,
 		    int32_t op_ret,
 		    int32_t op_errno,
-		    struct stat *buf)
+		    struct stat *buf,
+                    struct stat *preoldparent,
+                    struct stat *postoldparent,
+                    struct stat *prenewparent,
+                    struct stat *postnewparent)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
@@ -242,7 +202,9 @@ map_link_cbk (call_frame_t *frame,
 		  int32_t op_ret,
 		  int32_t op_errno,
 		  inode_t *inode,
-		  struct stat *buf)
+                  struct stat *buf,
+                  struct stat *preparent,
+                  struct stat *postparent)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
@@ -291,14 +253,15 @@ map_writev_cbk (call_frame_t *frame,
 		    xlator_t *this,
 		    int32_t op_ret,
 		    int32_t op_errno,
-		    struct stat *stbuf)
+                    struct stat *prebuf,
+		    struct stat *postbuf)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
 	
-	map_itransform (this, prev->this, stbuf->st_ino, &stbuf->st_ino);
+	map_itransform (this, prev->this, postbuf->st_ino, &postbuf->st_ino);
 
-	STACK_UNWIND (frame, op_ret, op_errno, stbuf);
+	STACK_UNWIND (frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -319,9 +282,11 @@ map_fsync_cbk (call_frame_t *frame,
 		   void *cookie,
 		   xlator_t *this,
 		   int32_t op_ret,
-		   int32_t op_errno)
+		   int32_t op_errno,
+                   struct stat *prebuf,
+                   struct stat *postbuf)
 {
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND (frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -527,7 +492,9 @@ map_newentry_cbk (call_frame_t *frame,
 		  int32_t op_ret,
 		  int32_t op_errno,
 		  inode_t *inode,
-		  struct stat *buf)
+                  struct stat *buf,
+                  struct stat *preparent,
+                  struct stat *postparent)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
@@ -548,7 +515,9 @@ map_create_cbk (call_frame_t *frame,
 		int32_t op_errno,
 		fd_t *fd,
 		inode_t *inode,
-		struct stat *buf)
+		struct stat *buf,
+                struct stat *preparent,
+                struct stat *postparent)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
@@ -656,7 +625,8 @@ map_single_lookup_cbk (call_frame_t *frame,
 		       int32_t op_errno,
 		       inode_t *inode,
 		       struct stat *buf,
-		       dict_t *dict)
+                       dict_t *dict,
+                       struct stat *postparent)
 {
         call_frame_t *prev = NULL;
         prev  = cookie;
@@ -676,7 +646,8 @@ map_root_lookup_cbk (call_frame_t *frame,
                      int32_t op_errno,
                      inode_t *inode,
                      struct stat *buf,
-                     dict_t *dict)
+                     dict_t *dict,
+                     struct stat *postparent)
 {
 	int          callcnt = 0;
 	map_local_t *local = NULL;
@@ -779,9 +750,36 @@ map_single_readdir_cbk (call_frame_t *frame,
 }
 
 
+int32_t
+map_single_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                         int32_t op_ret, int32_t op_errno, gf_dirent_t *entries)
+{
+        call_frame_t *prev = NULL;
+	gf_dirent_t  *orig_entry = NULL;
+
+        prev  = cookie;
+
+	list_for_each_entry (orig_entry, &entries->list, list) {
+		map_itransform (this, prev->this, orig_entry->d_ino,
+				&orig_entry->d_ino);
+                orig_entry->d_stat.st_ino = orig_entry->d_ino;
+	}
+	STACK_UNWIND (frame, op_ret, op_errno, entries);
+	return 0;
+}
+
 int
 map_readdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		 int op_ret, int op_errno, gf_dirent_t *orig_entries)
+                 int op_ret, int op_errno, gf_dirent_t *orig_entries);
+
+int
+map_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                 int op_ret, int op_errno, gf_dirent_t *orig_entries);
+
+int
+map_generic_readdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                         int op_ret, int op_errno, gf_dirent_t *orig_entries,
+                         int whichop)
 {
 	map_local_t  *local = NULL;
 	gf_dirent_t   entries;
@@ -816,6 +814,8 @@ map_readdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		map_itransform (this, subvol, orig_entry->d_off,
 				&entry->d_off);
 		
+                if (whichop == GF_FOP_READDIRP)
+                        entry->d_stat.st_ino = entry->d_ino;
 		entry->d_type = orig_entry->d_type;
 		entry->d_len  = orig_entry->d_len;
 		
@@ -841,9 +841,14 @@ done:
 			goto unwind;
 		}
 
-		STACK_WIND (frame, map_readdir_cbk,
-			    next_subvol, next_subvol->fops->readdir,
-			    local->fd, local->size, 0);
+                if (whichop == GF_FOP_READDIR)
+                        STACK_WIND (frame, map_readdir_cbk, next_subvol,
+                                    next_subvol->fops->readdir, local->fd,
+                                    local->size, 0);
+                else
+                        STACK_WIND (frame, map_readdirp_cbk, next_subvol,
+                                    next_subvol->fops->readdirp, local->fd,
+                                    local->size, 0);
 		return 0;
 	}
 
@@ -864,6 +869,26 @@ unwind:
 }
 
 
+int
+map_readdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                 int op_ret, int op_errno, gf_dirent_t *orig_entries)
+{
+        map_generic_readdir_cbk (frame, cookie, this, op_ret, op_errno,
+                                 orig_entries, GF_FOP_READDIR);
+        return 0;
+}
+
+
+int
+map_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                  int op_ret, int op_errno, gf_dirent_t *orig_entries)
+{
+        map_generic_readdir_cbk (frame, cookie, this, op_ret, op_errno,
+                                 orig_entries, GF_FOP_READDIRP);
+        return 0;
+}
+
+
 /* Management operations */
 
 static int32_t
@@ -928,21 +953,22 @@ map_stat (call_frame_t *frame,
 	return 0;
 }
 
-
 int32_t
-map_chmod (call_frame_t *frame,
-	   xlator_t *this,
-	   loc_t *loc,
-	   mode_t mode)
+map_setattr (call_frame_t *frame,
+             xlator_t *this,
+             loc_t *loc,
+             struct stat *stbuf,
+             int32_t valid)
 {
 	int32_t op_errno = 1;
 	xlator_t *subvol   = NULL;
 
-        VALIDATE_OR_GOTO (frame, err);
-        VALIDATE_OR_GOTO (this, err);
-        VALIDATE_OR_GOTO (loc, err);
-        VALIDATE_OR_GOTO (loc->inode, err);
-        VALIDATE_OR_GOTO (loc->path, err);
+        GF_VALIDATE_OR_GOTO ("map", this, err);
+        GF_VALIDATE_OR_GOTO (this->name, frame, err);
+        GF_VALIDATE_OR_GOTO (this->name, loc, err);
+        GF_VALIDATE_OR_GOTO (this->name, loc->inode, err);
+        GF_VALIDATE_OR_GOTO (this->name, loc->path, err);
+        GF_VALIDATE_OR_GOTO (this->name, stbuf, err);
 
 	subvol = get_mapping_subvol_from_ctx (this, loc->inode);
 	if (!subvol) {
@@ -950,8 +976,8 @@ map_chmod (call_frame_t *frame,
 		goto err;
 	}
 
-	STACK_WIND (frame, map_chmod_cbk, subvol, 
-                    subvol->fops->chmod, loc, mode);
+	STACK_WIND (frame, map_setattr_cbk, subvol,
+		    subvol->fops->setattr, loc, stbuf, valid);
 	return 0;
  err:
 	STACK_UNWIND (frame, -1, op_errno, NULL, NULL);
@@ -960,18 +986,19 @@ map_chmod (call_frame_t *frame,
 }
 
 int32_t
-map_fchmod (call_frame_t *frame,
-	    xlator_t *this,
-	    fd_t *fd,
-	    mode_t mode)
+map_fsetattr (call_frame_t *frame,
+              xlator_t *this,
+              fd_t *fd,
+              struct stat *stbuf,
+              int32_t valid)
 {
 	int32_t op_errno = 1;
 	xlator_t *subvol   = NULL;
 
-        VALIDATE_OR_GOTO (frame, err);
-        VALIDATE_OR_GOTO (this, err);
-        VALIDATE_OR_GOTO (fd, err);
-        VALIDATE_OR_GOTO (fd->inode, err);
+        GF_VALIDATE_OR_GOTO ("map", this, err);
+        GF_VALIDATE_OR_GOTO (this->name, frame, err);
+        GF_VALIDATE_OR_GOTO (this->name, fd, err);
+        GF_VALIDATE_OR_GOTO (this->name, stbuf, err);
 
 	subvol = get_mapping_subvol_from_ctx (this, fd->inode);
 	if (!subvol) {
@@ -979,71 +1006,8 @@ map_fchmod (call_frame_t *frame,
 		goto err;
 	}
 
-	STACK_WIND (frame, map_fchmod_cbk, subvol,
-		    subvol->fops->fchmod, fd, mode);
-
-	return 0;
- err:
-	STACK_UNWIND (frame, -1, op_errno, NULL, NULL);
-
-	return 0;
-}
-
-int32_t
-map_chown (call_frame_t *frame,
-	   xlator_t *this,
-	   loc_t *loc,
-	   uid_t uid,
-	   gid_t gid)
-{
-	int32_t op_errno = 1;
-	xlator_t *subvol   = NULL;
-
-        VALIDATE_OR_GOTO (frame, err);
-        VALIDATE_OR_GOTO (this, err);
-        VALIDATE_OR_GOTO (loc, err);
-        VALIDATE_OR_GOTO (loc->inode, err);
-        VALIDATE_OR_GOTO (loc->path, err);
-
-	subvol = get_mapping_subvol_from_ctx (this, loc->inode);
-	if (!subvol) {
-		op_errno = EINVAL;
-		goto err;
-	}
-
-	STACK_WIND (frame, map_chown_cbk, subvol,
-		    subvol->fops->chown, loc, uid, gid);
-	return 0;
- err:
-	STACK_UNWIND (frame, -1, op_errno, NULL, NULL);
-
-	return 0;
-}
-
-int32_t
-map_fchown (call_frame_t *frame,
-	    xlator_t *this,
-	    fd_t *fd,
-	    uid_t uid,
-	    gid_t gid)
-{
-	int32_t op_errno = 1;
-	xlator_t *subvol   = NULL;
-
-        VALIDATE_OR_GOTO (frame, err);
-        VALIDATE_OR_GOTO (this, err);
-        VALIDATE_OR_GOTO (fd, err);
-        VALIDATE_OR_GOTO (fd->inode, err);
-
-	subvol = get_mapping_subvol_from_ctx (this, fd->inode);
-	if (!subvol) {
-		op_errno = EINVAL;
-		goto err;
-	}
-
-	STACK_WIND (frame, map_fchown_cbk, subvol,
-		    subvol->fops->fchown, fd, uid, gid);
-
+	STACK_WIND (frame, map_fsetattr_cbk, subvol,
+		    subvol->fops->fsetattr, fd, stbuf, valid);
 	return 0;
  err:
 	STACK_UNWIND (frame, -1, op_errno, NULL, NULL);
@@ -1112,37 +1076,6 @@ map_ftruncate (call_frame_t *frame,
 	return 0;
 }
 
-int32_t
-map_utimens (call_frame_t *frame,
-	     xlator_t *this,
-	     loc_t *loc,
-	     struct timespec tv[2])
-{
-	int32_t op_errno = 1;
-	xlator_t *subvol   = NULL;
-
-        VALIDATE_OR_GOTO (frame, err);
-        VALIDATE_OR_GOTO (this, err);
-        VALIDATE_OR_GOTO (loc, err);
-        VALIDATE_OR_GOTO (loc->inode, err);
-        VALIDATE_OR_GOTO (loc->path, err);
-
-	subvol = get_mapping_subvol_from_ctx (this, loc->inode);
-	if (!subvol) {
-		op_errno = EINVAL;
-		goto err;
-	}
-
-	STACK_WIND (frame, map_utimens_cbk, subvol,
-                    subvol->fops->utimens, loc, tv);
-
-	return 0;
- err:
-	STACK_UNWIND (frame, -1, op_errno, NULL, NULL);
-
-	return 0;
-}
-
 int32_t
 map_access (call_frame_t *frame,
 	    xlator_t *this,
@@ -1353,7 +1286,7 @@ int32_t
 map_open (call_frame_t *frame,
 	  xlator_t *this,
 	  loc_t *loc,
-	  int32_t flags, fd_t *fd)
+	  int32_t flags, fd_t *fd, int wbflags)
 {
 	int32_t op_errno = 1;
 	xlator_t *subvol   = NULL;
@@ -1371,7 +1304,7 @@ map_open (call_frame_t *frame,
 	}
 
 	STACK_WIND (frame, map_open_cbk, subvol, 
-                    subvol->fops->open, loc, flags, fd);
+                    subvol->fops->open, loc, flags, fd, wbflags);
 
 	return 0;
  err:
@@ -2359,11 +2292,8 @@ map_opendir (call_frame_t *frame,
 
 
 int32_t
-map_readdir (call_frame_t *frame,
-	     xlator_t *this,
-	     fd_t *fd,
-	     size_t size,
-	     off_t yoff)
+map_do_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
+                off_t yoff, int whichop)
 {
 	int32_t        op_errno = EINVAL;
 	xlator_t      *subvol = NULL;
@@ -2386,8 +2316,12 @@ map_readdir (call_frame_t *frame,
 	}
 	
 	/* Just one callback */
-	STACK_WIND (frame, map_single_readdir_cbk, subvol,
-		    subvol->fops->readdir, fd, size, yoff);
+        if (whichop == GF_FOP_READDIR)
+                STACK_WIND (frame, map_single_readdir_cbk, subvol,
+                            subvol->fops->readdir, fd, size, yoff);
+        else
+                STACK_WIND (frame, map_single_readdirp_cbk, subvol,
+                            subvol->fops->readdirp, fd, size, yoff);
 
 	return 0;
 
@@ -2411,8 +2345,12 @@ map_readdir (call_frame_t *frame,
 
 	map_deitransform (this, yoff, &xvol, (uint64_t *)&xoff);
 
-	STACK_WIND (frame, map_readdir_cbk, xvol, 
-                    xvol->fops->readdir, fd, size, xoff);
+        if (whichop == GF_FOP_READDIR)
+                STACK_WIND (frame, map_readdir_cbk, xvol, xvol->fops->readdir,
+                            fd, size, xoff);
+        else
+                STACK_WIND (frame, map_readdirp_cbk, xvol, xvol->fops->readdirp,
+                            fd, size, xoff);
 
 	return 0;
  err:
@@ -2422,6 +2360,24 @@ map_readdir (call_frame_t *frame,
 }
 
 
+
+int32_t
+map_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
+             off_t yoff)
+{
+        map_do_readdir (frame, this, fd, size, yoff, GF_FOP_READDIR);
+        return 0;
+}
+
+int32_t
+map_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
+              off_t yoff)
+{
+        map_do_readdir (frame, this, fd, size, yoff, GF_FOP_READDIRP);
+        return 0;
+}
+
+
 void
 fini (xlator_t *this)
 {
@@ -2557,12 +2513,7 @@ struct xlator_fops fops = {
 	.create      = map_create,
 
 	.stat        = map_stat,
-	.chmod       = map_chmod,
-	.chown       = map_chown,
-	.fchown      = map_fchown,
-	.fchmod      = map_fchmod,
 	.fstat       = map_fstat,
-	.utimens     = map_utimens,
 	.truncate    = map_truncate,
 	.ftruncate   = map_ftruncate,
 	.access      = map_access,
@@ -2581,6 +2532,7 @@ struct xlator_fops fops = {
 	.lk          = map_lk,
 	.opendir     = map_opendir,
 	.readdir     = map_readdir,
+	.readdirp    = map_readdirp,
 	.fsyncdir    = map_fsyncdir,
 	.symlink     = map_symlink,
 	.unlink      = map_unlink,
@@ -2597,6 +2549,8 @@ struct xlator_fops fops = {
 	.setdents    = map_setdents,
 	.getdents    = map_getdents,
 	.checksum    = map_checksum,
+        .setattr     = map_setattr,
+        .fsetattr    = map_fsetattr,
 };
 
 struct xlator_mops mops = {
