@@ -178,7 +178,8 @@ unify_sh_setdents_cbk (call_frame_t *frame,
 			unify_local_wipe (local);
 			
 			STACK_UNWIND (frame, local->op_ret, local->op_errno, 
-				      inode, &local->stbuf, local->dict);
+				      inode, &local->stbuf, local->dict,
+                                      &local->oldpostparent);
 			if (tmp_dict)
 				dict_unref (tmp_dict);
 		}
@@ -501,7 +502,7 @@ unify_sh_opendir_cbk (call_frame_t *frame,
 
 		/* This is lookup_cbk ()'s UNWIND. */
 		STACK_UNWIND (frame, local->op_ret, local->op_errno, inode,
-			      &local->stbuf, local->dict);
+			      &local->stbuf, local->dict, &local->oldpostparent);
 		if (tmp_dict)
 			dict_unref (tmp_dict);
 	}
@@ -539,9 +540,9 @@ unify_sh_checksum_cbk (call_frame_t *frame,
 		if (op_ret >= 0) {
 			if (NS(this) == (xlator_t *)cookie) {
 				memcpy (local->sh_struct->ns_file_checksum, 
-					file_checksum, ZR_FILENAME_MAX);
+					file_checksum, NAME_MAX);
 				memcpy (local->sh_struct->ns_dir_checksum, 
-					dir_checksum, ZR_FILENAME_MAX);
+					dir_checksum, NAME_MAX);
 			} else {
 				if (local->entry_count == 0) {
 					/* Initialize the dir_checksum to be 
@@ -552,12 +553,12 @@ unify_sh_checksum_cbk (call_frame_t *frame,
                                         /* Using 'entry_count' as a flag */
 					local->entry_count = 1;
 					memcpy (local->sh_struct->dir_checksum,
-						dir_checksum, ZR_FILENAME_MAX);
+						dir_checksum, NAME_MAX);
 				}
 
 				/* Reply from the storage nodes */
 				for (index = 0; 
-				     index < ZR_FILENAME_MAX; index++) {
+				     index < NAME_MAX; index++) {
 					/* Files should be present in
 					   only one node */
 					local->sh_struct->file_checksum[index] ^= file_checksum[index];
@@ -573,7 +574,7 @@ unify_sh_checksum_cbk (call_frame_t *frame,
 	UNLOCK (&frame->lock);
 
 	if (!callcnt) {
-		for (index = 0; index < ZR_FILENAME_MAX ; index++) {
+		for (index = 0; index < NAME_MAX ; index++) {
 			if (local->sh_struct->file_checksum[index] != 
 			    local->sh_struct->ns_file_checksum[index]) {
 				local->failed = 1;
@@ -628,7 +629,7 @@ unify_sh_checksum_cbk (call_frame_t *frame,
 			      local->op_errno,
 			      inode,
 			      &local->stbuf,
-			      local->dict);
+			      local->dict, &local->oldpostparent);
 		if (tmp_dict)
 			dict_unref (tmp_dict);
 	}
@@ -1038,9 +1039,9 @@ unify_bgsh_checksum_cbk (call_frame_t *frame,
 		if (op_ret >= 0) {
 			if (NS(this) == (xlator_t *)cookie) {
 				memcpy (local->sh_struct->ns_file_checksum, 
-					file_checksum, ZR_FILENAME_MAX);
+					file_checksum, NAME_MAX);
 				memcpy (local->sh_struct->ns_dir_checksum, 
-					dir_checksum, ZR_FILENAME_MAX);
+					dir_checksum, NAME_MAX);
 			} else {
 				if (local->entry_count == 0) {
 					/* Initialize the dir_checksum to be 
@@ -1051,12 +1052,12 @@ unify_bgsh_checksum_cbk (call_frame_t *frame,
 					/* Using 'entry_count' as a flag */
 					local->entry_count = 1; 
 					memcpy (local->sh_struct->dir_checksum,
-						dir_checksum, ZR_FILENAME_MAX);
+						dir_checksum, NAME_MAX);
 				}
 
 				/* Reply from the storage nodes */
 				for (index = 0; 
-				     index < ZR_FILENAME_MAX; index++) {
+				     index < NAME_MAX; index++) {
 					/* Files should be present in only 
 					   one node */
 					local->sh_struct->file_checksum[index] ^= file_checksum[index];
@@ -1072,7 +1073,7 @@ unify_bgsh_checksum_cbk (call_frame_t *frame,
 	UNLOCK (&frame->lock);
 
 	if (!callcnt) {
-		for (index = 0; index < ZR_FILENAME_MAX ; index++) {
+		for (index = 0; index < NAME_MAX ; index++) {
 			if (local->sh_struct->file_checksum[index] != 
 			    local->sh_struct->ns_file_checksum[index]) {
 				local->failed = 1;
@@ -1215,7 +1216,8 @@ zr_unify_self_heal (call_frame_t *frame,
 		      local->op_errno,
 		      tmp_inode,
 		      &local->stbuf,
-		      local->dict);
+		      local->dict,
+                      &local->oldpostparent);
 
 	if (tmp_dict)
 		dict_unref (tmp_dict);
