@@ -28,6 +28,14 @@
 #include "defaults.h"
 #include "common-utils.h"
 
+#ifndef MAX_IOVEC
+#define MAX_IOVEC 16
+#endif
+
+#define GF_DISK_USAGE_STATBUF(blocks) (blocks * 512)
+#define GF_DISK_FILE_USAGE(preblocks,postblocks) ((postblocks - preblocks) * 512)
+#define GF_DISK_DIRECTORY_USAGE(preblocks,postblocks) ((postblocks - preblocks) * 512)
+
 struct quota_local {
 	struct stat    stbuf;
 	inode_t       *inode;
@@ -35,7 +43,7 @@ struct quota_local {
 	fd_t          *fd;
 	off_t          offset;
 	int32_t        count;
-	struct iovec  *vector;
+	struct iovec   vector[MAX_IOVEC];
 	struct iobref *iobref;
 	loc_t          loc;
 };
@@ -52,6 +60,8 @@ struct quota_priv {
 	uint32_t   current_free_disk;          /* current free disk space available, in % */
 	uint32_t   refresh_interval;           /* interval in seconds */
 	uint32_t   min_disk_last_updated_time; /* used for interval calculation */	
+
+	loc_t      root_loc;		     /* Store '/' loc_t to make xattr calls */
 };
 
 
@@ -71,14 +81,6 @@ quota_statvfs_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 }
 
 
-static void
-build_root_loc (xlator_t *this, loc_t *loc)
-{
-        memset (loc, 0, sizeof (*loc));
-	loc->path = "/";
-}
-
-
 void
 gf_quota_usage_subtract (xlator_t *this, size_t size)
 {
@@ -115,16 +117,17 @@ gf_quota_update_current_free_disk (xlator_t *this)
 {
 	call_frame_t *frame = NULL;
 	call_pool_t   *pool = NULL;
-	loc_t          loc;
+
+	struct quota_priv *priv = NULL;
 
 	pool  = this->ctx->pool;
 	frame = create_frame (this, pool);
   
-	build_root_loc (this, &loc);
+	priv = this->private;
 
 	STACK_WIND (frame, quota_statvfs_cbk,
 		    this->children->xlator,
-		    this->children->xlator->fops->statfs, &loc);
+		    this->children->xlator->fops->statfs, &(priv->root_loc));
 
 	return ;
 }
@@ -158,63 +161,23 @@ quota_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                     struct stat *postbuf)
 {
 	struct quota_priv *priv = this->private;
-	struct quota_local *local = NULL;
 
-	local = frame->local;
+	int space_freed = 0;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		gf_quota_usage_subtract (this, (local->stbuf.st_blocks -
-						postbuf->st_blocks) * 512);
-		loc_wipe (&local->loc);
+		space_freed = GF_DISK_FILE_USAGE (prebuf->st_blocks, postbuf->st_blocks);
+		gf_quota_usage_subtract (this, space_freed);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno, prebuf, postbuf);
+	STACK_UNWIND_STRICT (truncate, frame, op_ret, op_errno,
+                             prebuf, postbuf);
 	return 0;
 }
 
-
-int
-quota_truncate_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-			 int32_t op_ret, int32_t op_errno, struct stat *buf)
-{
-	struct quota_local *local = NULL;
-	struct quota_priv  *priv = NULL;
-
-	priv = this->private;
-	local = frame->local;
-
-	if (op_ret >= 0) {
-		local->stbuf = *buf;
-	}
-
-	STACK_WIND (frame, quota_truncate_cbk,
-		    FIRST_CHILD (this), FIRST_CHILD (this)->fops->truncate,
-		    &local->loc, local->offset);
-	return 0;
-}
-
-
 int
 quota_truncate (call_frame_t *frame, xlator_t *this,
 		loc_t *loc, off_t offset)
 {
-	struct quota_local *local = NULL;
-	struct quota_priv  *priv = NULL;
-
-	priv = this->private;
-
-	if (priv->disk_usage_limit) {
-		local = CALLOC (1, sizeof (struct quota_local));
-		frame->local  = local;
-
-		loc_copy (&local->loc, loc);
-		local->offset = offset;
-
-		STACK_WIND (frame, quota_truncate_stat_cbk,
-			    FIRST_CHILD(this),
-			    FIRST_CHILD(this)->fops->stat, loc);
-		return 0;
-	}
 
 	STACK_WIND (frame, quota_truncate_cbk,
 		    FIRST_CHILD(this),
@@ -230,39 +193,17 @@ quota_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                      struct stat *postbuf)
 {
 	struct quota_priv  *priv = NULL;
-	struct quota_local *local = NULL;
+	int space_freed = 0;
 
-	local = frame->local;
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		gf_quota_usage_subtract (this, (local->stbuf.st_blocks -
-						postbuf->st_blocks) * 512);
-		fd_unref (local->fd);
+		space_freed = GF_DISK_FILE_USAGE (prebuf->st_blocks, postbuf->st_blocks);
+		gf_quota_usage_subtract (this, space_freed);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno, prebuf, postbuf);
-	return 0;
-}
-
-
-int
-quota_ftruncate_fstat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-			   int32_t op_ret, int32_t op_errno, struct stat *buf)
-{
-	struct quota_local *local = NULL;
-	struct quota_priv  *priv = NULL;
-
-	priv = this->private;
-	local = frame->local;
-
-	if (op_ret >= 0) {
-		local->stbuf = *buf;
-	}
-
-	STACK_WIND (frame, quota_ftruncate_cbk,
-		    FIRST_CHILD (this), FIRST_CHILD (this)->fops->ftruncate,
-		    local->fd, local->offset);
+	STACK_UNWIND_STRICT (ftruncate, frame, op_ret, op_errno,
+                             prebuf, postbuf);
 	return 0;
 }
 
@@ -271,24 +212,6 @@ int
 quota_ftruncate (call_frame_t *frame, xlator_t *this,
 		 fd_t *fd, off_t offset)
 {
-	struct quota_local *local = NULL;
-	struct quota_priv  *priv = NULL;
-
-
-	priv = this->private;
-
-	if (priv->disk_usage_limit) {
-		local = CALLOC (1, sizeof (struct quota_local));
-		frame->local  = local;
-
-		local->fd = fd_ref (fd);
-		local->offset = offset;
-
-		STACK_WIND (frame, quota_ftruncate_fstat_cbk,
-			    FIRST_CHILD(this),
-			    FIRST_CHILD(this)->fops->fstat, fd);
-		return 0;
-	}
 
 	STACK_WIND (frame, quota_ftruncate_cbk,
 		    FIRST_CHILD(this),
@@ -305,14 +228,19 @@ quota_mknod_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  struct stat *postparent)
 {
 	struct quota_priv *priv = NULL;
+	int space_used = 0;
 
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		gf_quota_usage_add (this, buf->st_blocks * 512);
+		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) +
+			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
+						 postparent->st_blocks);
+		gf_quota_usage_add (this, space_used);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno, inode, buf);
+	STACK_UNWIND_STRICT (mknod, frame, op_ret, op_errno, inode, buf,
+                             preparent, postparent);
 	return 0;
 }
 
@@ -329,7 +257,8 @@ quota_mknod (call_frame_t *frame, xlator_t *this,
 		gf_log (this->name, GF_LOG_ERROR, 
 			"min-free-disk limit (%u) crossed, current available is %u",
 			priv->min_free_disk_limit, priv->current_free_disk);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL);
+		STACK_UNWIND_STRICT (mknod, frame, -1, ENOSPC, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
 	}
 
@@ -337,7 +266,8 @@ quota_mknod (call_frame_t *frame, xlator_t *this,
 		gf_log (this->name, GF_LOG_ERROR, 
 			"Disk usage limit (%"PRIu64") crossed, current usage is %"PRIu64"",
 			priv->disk_usage_limit, priv->current_disk_usage);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL);
+		STACK_UNWIND_STRICT (mknod, frame, -1, ENOSPC, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
         }
 
@@ -356,14 +286,20 @@ quota_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  struct stat *postparent)
 {
 	struct quota_priv *priv = NULL;
+	int space_used = 0;
 
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		gf_quota_usage_subtract (this, buf->st_blocks * 512);
+
+		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) +
+			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
+						 postparent->st_blocks);
+		gf_quota_usage_add (this, space_used);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno, inode, buf);
+	STACK_UNWIND_STRICT (mkdir, frame, op_ret, op_errno, inode, buf,
+                             preparent, postparent);
 	return 0;
 }
 
@@ -379,7 +315,8 @@ quota_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode)
 		gf_log (this->name, GF_LOG_ERROR, 
 			"min-free-disk limit (%u) crossed, current available is %u",
 			priv->min_free_disk_limit, priv->current_free_disk);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL);
+		STACK_UNWIND_STRICT (mkdir, frame, -1, ENOSPC, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
 		
 	}
@@ -388,7 +325,8 @@ quota_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode)
 		gf_log (this->name, GF_LOG_ERROR, 
 			"Disk usage limit (%"PRIu64") crossed, current usage is %"PRIu64"",
 			priv->disk_usage_limit, priv->current_disk_usage);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL);
+		STACK_UNWIND_STRICT (mkdir, frame, -1, ENOSPC, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
         }
 
@@ -407,18 +345,21 @@ quota_unlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                   struct stat *postparent)
 {
 	struct quota_local *local = NULL;
+	int space_freed = 0;
 
 	local = frame->local;
 
 	if (local) {
 		if (op_ret >= 0) {
-			gf_quota_usage_subtract (this,
-						 local->stbuf.st_blocks * 512);
+			space_freed = GF_DISK_USAGE_STATBUF (local->stbuf.st_blocks) +
+				GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
+							 postparent->st_blocks);
+			gf_quota_usage_subtract (this, space_freed);
 		}
 		loc_wipe (&local->loc);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND_STRICT (unlink, frame, op_ret, op_errno, preparent, postparent);
 	return 0;
 }
 
@@ -482,17 +423,21 @@ quota_rmdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  struct stat *postparent)
 {
 	struct quota_local *local = NULL;
+	int space_freed = 0;
 
 	local = frame->local;
 
 	if (local) {
 		if (op_ret >= 0) {
-			gf_quota_usage_subtract (this, local->stbuf.st_blocks * 512);
+			space_freed = GF_DISK_USAGE_STATBUF (local->stbuf.st_blocks) +
+				GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
+							 postparent->st_blocks);
+			gf_quota_usage_subtract (this, space_freed);
 		}
 		loc_wipe (&local->loc);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno);
+	STACK_UNWIND_STRICT (rmdir, frame, op_ret, op_errno, preparent, postparent);
 	return 0;
 }
 
@@ -553,14 +498,19 @@ quota_symlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                    struct stat *postparent)
 {
 	struct quota_priv *priv = NULL;
+	int space_used = 0;
 
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		gf_quota_usage_add (this, buf->st_blocks * 512);
+		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) +
+			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
+						 postparent->st_blocks);
+		gf_quota_usage_add (this, space_used);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno, inode, buf);
+	STACK_UNWIND_STRICT (symlink, frame, op_ret, op_errno, inode, buf,
+                             preparent, postparent);
 	return 0;
 }
 
@@ -577,7 +527,8 @@ quota_symlink (call_frame_t *frame, xlator_t *this,
 		gf_log (this->name, GF_LOG_ERROR, 
 			"min-free-disk limit (%u) crossed, current available is %u",
 			priv->min_free_disk_limit, priv->current_free_disk);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL);
+		STACK_UNWIND_STRICT (symlink, frame, -1, ENOSPC, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
 		
 	}
@@ -585,7 +536,8 @@ quota_symlink (call_frame_t *frame, xlator_t *this,
 		gf_log (this->name, GF_LOG_ERROR, 
 			"Disk usage limit (%"PRIu64") crossed, current usage is %"PRIu64"",
 			priv->disk_usage_limit, priv->current_disk_usage);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL);
+		STACK_UNWIND_STRICT (symlink, frame, -1, ENOSPC, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
         }
 
@@ -606,13 +558,19 @@ quota_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	struct quota_priv *priv = this->private;
 	int                ret = 0;
 
-	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		gf_quota_usage_add (this, buf->st_blocks * 512);
+	int space_used = 0;
+
+	if ((op_ret >= 0) && priv->current_disk_usage) {
+		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) + 
+			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks, 
+						 postparent->st_blocks);
+		gf_quota_usage_add (this, space_used);
 
 		ret = fd_ctx_set (fd, this, 1);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno, fd, inode, buf);
+	STACK_UNWIND_STRICT (create, frame, op_ret, op_errno, fd, inode, buf,
+                             preparent, postparent);
 	return 0;
 }
 
@@ -629,7 +587,8 @@ quota_create (call_frame_t *frame, xlator_t *this,
 		gf_log (this->name, GF_LOG_ERROR, 
 			"min-free-disk limit (%u) crossed, current available is %u",
 			priv->min_free_disk_limit, priv->current_free_disk);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL, NULL);
+		STACK_UNWIND_STRICT (create, frame, -1, ENOSPC, NULL, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
 		
 	}
@@ -637,7 +596,8 @@ quota_create (call_frame_t *frame, xlator_t *this,
 		gf_log (this->name, GF_LOG_ERROR, 
 			"Disk usage limit (%"PRIu64") crossed, current usage is %"PRIu64"",
 			priv->disk_usage_limit, priv->current_disk_usage);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL, NULL, NULL);
+		STACK_UNWIND_STRICT (create, frame, -1, ENOSPC, NULL, NULL, NULL,
+                                     NULL, NULL);
 		return 0;
         }
 
@@ -658,7 +618,7 @@ quota_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	if (op_ret >= 0)
 		ret = fd_ctx_set (fd, this, 1);
 
-	STACK_UNWIND (frame, op_ret, op_errno, fd);
+	STACK_UNWIND_STRICT (open, frame, op_ret, op_errno, fd);
 	return 0;
 }
 
@@ -690,13 +650,13 @@ quota_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	if (priv->disk_usage_limit) {
 		if (op_ret >= 0) { 
 			gf_quota_usage_add (this, (postbuf->st_blocks -
-						   local->stbuf.st_blocks) * 512);
+						   prebuf->st_blocks) * 512);
 		}
 		fd_unref (local->fd);
 		iobref_unref (local->iobref);
 	}
 
-	STACK_UNWIND (frame, op_ret, op_errno, prebuf, postbuf);
+	STACK_UNWIND_STRICT (writev, frame, op_ret, op_errno, prebuf, postbuf);
 	return 0;
 }
 
@@ -720,7 +680,8 @@ quota_writev_fstat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			if (iovlen > (buf->st_blksize - (buf->st_size % buf->st_blksize))) {
 				fd_unref (local->fd);
 				iobref_unref (local->iobref);
-				STACK_UNWIND (frame, -1, ENOSPC, NULL);
+				STACK_UNWIND_STRICT (writev, frame, -1, ENOSPC,
+                                                     NULL, NULL);
 				return 0;
 			}
 		}
@@ -743,7 +704,8 @@ quota_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
               struct iobref *iobref)
 {
 	struct quota_local *local = NULL;
-	struct quota_priv  *priv = NULL;
+	struct quota_priv  *priv  = NULL;
+        int                 i     = 0;
 
 	priv = this->private;
 
@@ -751,7 +713,8 @@ quota_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
 		gf_log (this->name, GF_LOG_ERROR, 
 			"min-free-disk limit (%u) crossed, current available is %u",
 			priv->min_free_disk_limit, priv->current_free_disk);
-		STACK_UNWIND (frame, -1, ENOSPC, NULL);
+		STACK_UNWIND_STRICT (writev, frame, -1, ENOSPC,
+                                     NULL, NULL);
 		return 0;
 	}
 
@@ -759,8 +722,12 @@ quota_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
 		local = CALLOC (1, sizeof (struct quota_local));
 		local->fd     = fd_ref (fd);
 		local->iobref = iobref_ref (iobref);
-		local->vector = vector;
-		local->count  = count;
+                for (i = 0; i < count; i++) {
+                        local->vector[i].iov_base = vector[i].iov_base;
+                        local->vector[i].iov_len = vector[i].iov_len;
+                }
+
+                local->count = count;
 		local->offset = off;
 		frame->local  = local;
 
@@ -797,12 +764,19 @@ int
 quota_setxattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		    int32_t op_ret, int32_t op_errno)
 {
+        dict_t            *dict = NULL;
+
 	if (op_ret == -1) {
 		gf_log (this->name, GF_LOG_CRITICAL, 
 			"failed to set the disk-usage value: %s",
 			strerror (op_errno));
 	} 
 
+        if (cookie) {
+                dict = (dict_t *) cookie;
+                dict_unref (dict);
+        }
+
 	STACK_DESTROY (frame->root);
 	return 0;
 }
@@ -837,7 +811,7 @@ quota_statfs_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		statvfs->f_bfree = statvfs->f_bavail = 0;
 
 unwind:
-	STACK_UNWIND (frame, op_ret, op_errno, statvfs);
+	STACK_UNWIND_STRICT (statfs, frame, op_ret, op_errno, statvfs);
 	return 0;
 }
 
@@ -883,16 +857,17 @@ gf_quota_get_disk_usage (xlator_t *this)
 {
 	call_frame_t *frame = NULL;
 	call_pool_t  *pool = NULL;
-	loc_t         loc;
+
+	struct quota_priv *priv = NULL;
 
 	pool = this->ctx->pool;
 	frame = create_frame (this, pool);
-	build_root_loc (this, &loc);
+	priv = this->private;
 
 	STACK_WIND (frame, quota_getxattr_cbk,
 		    this->children->xlator,
 		    this->children->xlator->fops->getxattr,
-		    &loc,
+		    &(priv->root_loc),
 		    "trusted.glusterfs-quota-du");
 	return ;
 }
@@ -904,20 +879,22 @@ gf_quota_cache_sync (xlator_t *this)
 	struct quota_priv *priv = NULL;
 	call_frame_t      *frame = NULL;
 	dict_t            *dict = get_new_dict ();
-	loc_t              loc;
+
 
 
 	priv = this->private;
-	build_root_loc (this, &loc);
 
 	frame = create_frame (this, this->ctx->pool);
 	dict_set (dict, "trusted.glusterfs-quota-du", 
 		  data_from_uint64 (priv->current_disk_usage));
 
-	STACK_WIND (frame, quota_setxattr_cbk,
-		    this->children->xlator,
-		    this->children->xlator->fops->setxattr,
-		    &loc, dict, 0);
+        dict_ref (dict);
+
+	STACK_WIND_COOKIE (frame, quota_setxattr_cbk,
+                           (void *) (dict_t *) dict,
+                           this->children->xlator,
+                           this->children->xlator->fops->setxattr,
+                           &(priv->root_loc), dict, 0);
 }
 
 
@@ -937,25 +914,59 @@ notify (xlator_t *this,
 	void *data,
 	...)
 {
-	struct quota_priv *priv = this->private;
-	
-	switch (event)
-	{
-	case GF_EVENT_CHILD_UP:
-		if (priv->only_first_time) {
-			priv->only_first_time = 0;
-			if (priv->disk_usage_limit) {
-				gf_quota_get_disk_usage (this);
-			}
-		}
-	default:
-		default_notify (this, event, data);
-		break;
-	}
-
+	default_notify (this, event, data);
 	return 0;
 }
 
+int32_t
+quota_lookup_cbk (call_frame_t *frame,
+		    void *cookie,
+		    xlator_t *this,
+		    int32_t op_ret,
+		    int32_t op_errno,
+		    inode_t *inode,
+		    struct stat *buf,
+                    dict_t *dict,
+                    struct stat *postparent)
+{
+	STACK_UNWIND (frame,
+		      op_ret,
+		      op_errno,
+		      inode,
+		      buf,
+                      dict,
+                      postparent);
+	return 0;
+}
+
+
+int32_t
+quota_lookup (call_frame_t *frame,
+		xlator_t *this,
+		loc_t *loc,
+		dict_t *xattr_req)
+{
+	struct quota_priv *priv = NULL;
+
+	priv = this->private;
+
+	if (priv->only_first_time) {
+		if (strcmp (loc->path, "/") == 0) {
+			loc_copy(&(priv->root_loc), loc); 
+			priv->only_first_time = 0;
+			if (priv->disk_usage_limit)
+				gf_quota_get_disk_usage (this);
+		}
+	}
+
+	STACK_WIND (frame,
+		    quota_lookup_cbk,
+		    FIRST_CHILD(this),
+		    FIRST_CHILD(this)->fops->lookup,
+		    loc,
+		    xattr_req);
+	return 0;
+ }
 
 int32_t 
 init (xlator_t *this)
@@ -1036,6 +1047,7 @@ fini (xlator_t *this)
 struct xlator_fops fops = {
 	.create      = quota_create,
 	.open        = quota_open,
+	.lookup	     = quota_lookup,
 	.truncate    = quota_truncate,
 	.ftruncate   = quota_ftruncate,
 	.writev      = quota_writev,
