@@ -24,6 +24,7 @@
 
 #include "io-cache.h"
 
+extern int ioc_log2_page_size;
 
 /*
  * str_to_ptr - convert a string to pointer
@@ -48,8 +49,14 @@ str_to_ptr (char *string)
 char *
 ptr_to_str (void *ptr)
 {
+        int   ret = 0;
 	char *str = NULL;
-	asprintf (&str, "%p", ptr);
+	ret = asprintf (&str, "%p", ptr);
+        if (-1 == ret) {
+                gf_log ("ioc", GF_LOG_ERROR, 
+                        "asprintf failed while converting ptr to str");
+                return NULL;
+        }
 	return str;
 }
 
@@ -147,16 +154,20 @@ ioc_inode_wakeup (call_frame_t *frame, ioc_inode_t *ioc_inode,
 ioc_inode_t *
 ioc_inode_update (ioc_table_t *table, inode_t *inode, uint32_t weight)
 {
-	ioc_inode_t *ioc_inode = NULL;
-        
+	ioc_inode_t     *ioc_inode   = NULL;
+        unsigned long    no_of_pages = 0;
+
         ioc_inode = CALLOC (1, sizeof (ioc_inode_t));
-	ERR_ABORT (ioc_inode);
+        if (ioc_inode == NULL) {
+                goto out;
+        }
   
 	ioc_inode->table = table;
- 
-	/* initialize the list for pages */
-	INIT_LIST_HEAD (&ioc_inode->pages);
-	INIT_LIST_HEAD (&ioc_inode->page_lru);
+
+        no_of_pages = (table->cache_size / table->page_size)
+                + ((table->cache_size % table->page_size) ? 1 : 0);
+
+	INIT_LIST_HEAD (&ioc_inode->cache.page_lru);
 
 	ioc_table_lock (table);
 
@@ -173,6 +184,7 @@ ioc_inode_update (ioc_table_t *table, inode_t *inode, uint32_t weight)
 	pthread_mutex_init (&ioc_inode->inode_lock, NULL);
 	ioc_inode->weight = weight;
   
+out:
 	return ioc_inode;
 }
 
@@ -198,6 +210,7 @@ ioc_inode_destroy (ioc_inode_t *ioc_inode)
 	ioc_table_unlock (table);
   
 	ioc_inode_flush (ioc_inode);
+        rbthash_table_destroy (ioc_inode->cache.page_table);
 
 	pthread_mutex_destroy (&ioc_inode->inode_lock);
 	free (ioc_inode);
