@@ -109,7 +109,7 @@ resolve_loc_touchup (call_frame_t *frame)
         loc     = state->loc_now;
 
         if (!loc->path) {
-                if (loc->parent) {
+                if (loc->parent && resolve->bname) {
                         ret = inode_path (loc->parent, resolve->bname, &path);
                 } else if (loc->inode) {
                         ret = inode_path (loc->inode, NULL, &path);
@@ -532,7 +532,7 @@ server_resolve_fd (call_frame_t *frame)
 
         if (!state->fd) {
                 resolve->op_ret   = -1;
-                resolve->op_errno = EBADF;
+                resolve->op_errno = EBADFD;
         }
 
         server_resolve_all (frame);
