@@ -29,6 +29,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <dirent.h>
+#include <time.h>
 
 #ifdef linux
 #ifdef __GLIBC__
@@ -59,18 +60,30 @@ struct posix_fd {
 	int32_t flags;   /* flags for open/creat      */
 	char *  path;    /* used by setdents/getdents */
 	DIR *   dir;     /* handle returned by the kernel */
+        int     flushwrites;
+        struct list_head list; /* to add to the janitor list */
 };
 
+
 struct posix_private {
 	char   *base_path;
 	int32_t base_path_length;
 
+        gf_lock_t lock;
+
+        char   hostname[256];
         /* Statistics, provides activity of the server */
 	struct xlator_stats stats; 
   
 	struct timeval prev_fetch_time;
 	struct timeval init_time;
 
+        time_t last_landfill_check;
+        int32_t janitor_sleep_duration;
+        struct list_head janitor_fds;
+        pthread_cond_t janitor_cond;
+        pthread_mutex_t janitor_lock;
+
 	int32_t max_read;            /* */
 	int32_t max_write;           /* */
 	int64_t interval_read;      /* Used to calculate the max_read value */
@@ -103,6 +116,17 @@ struct posix_private {
 
         int             num_devices_to_span;
         dev_t          *st_device;
+
+/* a global generation number sequence is used to assign generation numbers in 
+   sequence.
+*/
+        uint64_t        gen_seq;
+        gf_lock_t       gen_lock;
+
+/* janitor thread which cleans up /.trash (created by replicate) */
+        pthread_t       janitor;
+        gf_boolean_t    janitor_present;
+        char *          trash_path;
 };
 
 #define POSIX_BASE_PATH(this) (((struct posix_private *)this->private)->base_path)
